/*
 * fileops.c -- file operations.
 */

# include	<stdio.h>
# include	<ctype.h>

# include	<GestaltEqu.h>
# include	<Folders.h>
# include	<pascal.h>

# include	"TransSkel.h"

# include	"rtf.h"
# include	"rtf-mac.h"
# include	"MyStandardFile.h"


/*
 * Test whether or not A/UX is running.  This affects the path component
 * delimiter that's used to construct full pathnames.
 */

static Boolean
HaveAux ()
{
long	result;

	return (SkelQuery (skelQHasGestalt)
		&& Gestalt (gestaltAUXVersion, &result) == noErr);
}


/*
 * Return the correct pathname component separator character.
 */

short
PathSepChar (void)
{
	return (HaveAux () ? '/' : ':');
}



/*
 * Convert dirID, vRefNum pair to full directory pathname.
 * Path is returned as a Pascal string.
 */

void
MakeDirPath (long dirID, short vRefNum, StringPtr path)
{
CInfoPBRec	pbBlock;
Str31		dirName;
short		n;
StringPtr	delim;

	delim = "\pX";
	delim[1] = PathSepChar ();

	path[0] = '\0';						/* start with empty string */
	pbBlock.dirInfo.ioDrParID = dirID;
	pbBlock.dirInfo.ioNamePtr = dirName;
	for (;;)
	{
		pbBlock.dirInfo.ioVRefNum = vRefNum;
		pbBlock.dirInfo.ioFDirIndex = -1;
		pbBlock.dirInfo.ioDrDirID = pbBlock.dirInfo.ioDrParID;
		(void) PBGetCatInfo (&pbBlock, false);
		if (path[0] != '\0')
			PStrInsert (path, delim);
		PStrInsert (path, dirName);
		if (pbBlock.dirInfo.ioDrDirID == 2)
			break;
	}
}


/*
 * Given a directory pathname and basename as Pascal strings, construct
 * the full pathname as a C string.
 */

void
MakeCFilePath (StringPtr dir, StringPtr basename, char *file)
{
StringPtr	delim;

	delim = "\pX";
	delim[1] = PathSepChar ();

	PStrCpy ((StringPtr) file, dir);
	PStrCat ((StringPtr) file, delim);
	PStrCat ((StringPtr) file, basename);
	(void) PtoCstr ((StringPtr) file);
}


/*
 * Return pointer to the full pathname of the current application's
 * folder (as a C string).  The path is computed on the first call,
 * then remembered for future calls.  This means the function will
 * fail if the user moves the application or renames one of its enclosing
 * folders.
 */

char *
AppFolderPath (void)
{
FCBPBRec	homeDir;
Str31		dirName;
short		pathRefNum;
Handle		appParam;
Str255		str;
static char	*path = (char *) NULL;

	if (path == (char *) NULL)		/* compute only if haven't already */
	{
		GetAppParms (str, &pathRefNum, &appParam);
		homeDir.ioRefNum = pathRefNum;
		homeDir.ioFCBIndx = 0;
		homeDir.ioNamePtr = dirName;
		PBGetFCBInfoSync (&homeDir);
		MakeDirPath (homeDir.ioFCBParID, homeDir.ioFCBVRefNum, str);
		path = RTFStrSave (PtoCstr (str));
	}
	return (path);
}


/*
 * Return pointer to full pathname of "RTF Tools" folder, located in
 * the Preferences folder (as a C string).  The path is computed on
 * the first call, then remembered for future calls.  This means the
 * function will  fail if the user moves the system folder.
 */

char *
PrefsFolderPath (void)
{
short	vRefNum;
long	dirID;
OSErr	err;
Str255	str;
StringPtr	delim;
static char	*path = (char *) NULL;

	delim = "\pX";
	delim[1] = PathSepChar ();

	if (path == (char *) NULL)		/* compute only if haven't already */
	{
		err = FindFolder (kOnSystemDisk, kPreferencesFolderType,
							kDontCreateFolder, &vRefNum, &dirID);
		if (err == noErr)
		{
			MakeDirPath (dirID, vRefNum, str);
			PStrCat (str, delim);
			PStrCat (str, rtfLibFolderName);
			path = RTFStrSave (PtoCstr (str));
		}
	}
	return (path);
}


/*
 * Try to open a library file.
 *
 * Look first in application folder, then in the RTF Tools folder that's
 * in the Preferences folder.
 *
 * Declared in non-prototyped fashion since it's a callback for the RTF library,
 * which is written without prototypes.
 */

FILE *
MacOpenLibFile (file, mode)
char	*file;
char	*mode;
{
FILE	*f;
char	buf[rtfBufSiz];
char	*p;

	(void) strcpy (buf, AppFolderPath ());
	(void) strcat (buf, ":");
	(void) strcat (buf, file);
	if ((f = fopen (buf, mode)) != (FILE *) NULL)
		return (f);
	p = PrefsFolderPath ();
	if (p != (char *) NULL)
	{
		(void) strcpy (buf, p);
		(void) strcat (buf, ":");
		(void) strcat (buf, file);
		if ((f = fopen (buf, mode)) != (FILE *) NULL)
			return (f);
	}
	return ((FILE *) NULL);
}


/*
 * Run GetFile dialog to allow the user to select a file for processing.
 * Add it to the document queue.  The output file name is derived from the
 * input name, so it's not solicited.
 *
 * Return true if a file was selected, false otherwise.
 */

Boolean
SelectInputFile (FSSpec *fss, OSType type)
{
StandardFileReply	sfReply;
SFTypeList	typeList;

	typeList[0] = type;
	MyStandardGetFile((FileFilterProcPtr) nil, 1, typeList, &sfReply);
	SkelDoUpdates ();
	if (sfReply.sfGood)
		*fss = sfReply.sfFile;
	return (sfReply.sfGood);
}


/*
 * Run PutFile dialog to allow the user to select an output file.
 * Default name is in fss->name (so initialize it before calling!).
 * Default folder is specified by fss->parID/fss->vRefNum, unless
 * parent ID is zero (initialize before calling these, too).
 * fileType is a string that goes into the prompt, e.g., "\poutput"
 * or "\perror".
 *
 * Return true if a file was selected, false otherwise.
 */

Boolean
SelectOutputFile (FSSpec *fss, StringPtr fileType)
{
StandardFileReply	sfReply;
Str255	dir;
Str255	prompt;

	PStrCpy (prompt, "\pSelect ");
	PStrCat (prompt, fileType);
	PStrCat (prompt, "\p file:");
	sfReply.sfFile = *fss;
	MyStandardPutFile (prompt, fss->name, &sfReply);
	SkelDoUpdates ();
	if (sfReply.sfGood)
		*fss = sfReply.sfFile;
	return (sfReply.sfGood);
}


/*
 * Derive output filename from input filename.
 *
 * If the input name ends in inSuffix, strip it and add outSuffix to
 * get the output name.  If the input name doesn't end in inSuffix,
 * just add outPrefix.  Suppose inSuffix is ".in" and outSuffix is ".out".
 * Then you'd have:
 *    xxx.in -> xxx.out
 *    xxx -> xxx.out
 *    xxx.abc -> xxx.abc.out
 *
 * Comparisons for the input suffix are case insensitive.
 *
 * Strings are all Pascal strings.  Neither suffix is required to begin
 * with the dot (.), but it will be stripped from the input name if the
 * suffix is otherwise present, and will be added before the output suffix
 * if the suffix does not begin with a dot.
 */

void
MakeOutputFileName (StringPtr inName, StringPtr inSuffix,
							StringPtr outName, StringPtr outSuffix)
{
int	nLen, sLen;
int	i;
int	c1, c2;

	PStrCpy (outName, inName);
	nLen = outName[0];
	sLen = inSuffix[0];
	if (nLen >= sLen)			/* strip inSuffix if it's present */
	{
		for (i = 0; i < sLen; i++)
		{
			c1 = outName[nLen-i];
			c2 = inSuffix[sLen-i];
			if (isupper (c1))
				c1 = tolower (c1);
			if (isupper (c2))
				c2 = tolower (c2);
			if (c1 != c2)
				break;
		}
		if (i == sLen)			/* suffix present; strip it */
		{
			outName[0] -= sLen;
			/* strip dot, too, if present and suffix didn't include it */
			if (sLen > 0 && inSuffix[1] != '.')
			{
				if (outName[outName[0]] == '.')
					--outName[0];
			}
		}
	}
	/* add output suffix, including dot if not present in suffix string */
	if (outSuffix[0] > 0 && outSuffix[1] != '.')
		PStrCat (outName, "\p.");
	PStrCat (outName, outSuffix);
}
