/*
 * - Why does TransSkel lock up if no clobber proc is supplied?
 */

/*
 * statuswind.c -- routines to manage a status window showing the
 * current application status:
 * - Number of documents processed
 * - Number of documents pending/in-process
 * - Current input and output file
 * - Thermometer showing progress of current file translation
 * - Cancel button for terminating current translation
 *
 * 03 May 94
 * - Added Cancel button to window for terminating current translation.
 * It's only used if the button resource is found in the resource file.
 */

# include	<stdio.h>
# include	<pascal.h>

# include	"TransSkel.h"
# include	"TransDisplay.h"

# include	"rtf-mac.h"

# define	normalHilite		0
# define	dimHilite			255

# define	processedItem		0
# define	pendingItem			1
# define	inputItem			2
# define	outputItem			3
# define	maxDispItem			4

# define	processedTitle		"\pDocuments processed:"
# define	pendingTitle		"\pDocuments pending/in process:"
# define	inputTitle			"\pInput:"
# define	outputTitle			"\pOutput:"

# define	titleHOffset		10
# define	processedVOffset	20
# define	pendingVOffset		40
# define	inputVOffset		60
# define	outputVOffset		80


typedef	struct	DispItem	DispItem;

struct DispItem
{
	StringPtr	title;			/* item title */
	Str63		value;			/* current item value */
	short		vOffset;		/* item vertical offset */
};


static	DispItem	dispItem [maxDispItem] =
{
	{
		processedTitle,
		"\p",
		processedVOffset
	},
	{
		pendingTitle,
		"\p",
		pendingVOffset
	},
	{
		inputTitle,
		"\p",
		inputVOffset
	},
	{
		outputTitle,
		"\p",
		outputVOffset
	}
};

static WindowPtr	statusWind = (WindowPtr) nil;
static ControlHandle	cancelBtn = (ControlHandle) nil;
static short	percent;
static Rect		barRect;
static Boolean	useColorBar;	/* whether or not to draw bar in color */
static short	processed = -1;
static short	pending = -1;

/*
 * Standard meter bar dark gray foreground and lavender background colors.
 */

static RGBColor	barForeGnd = { 0x4444, 0x4444, 0x4444 };
static RGBColor	barBackGnd = { 0xcccc, 0xcccc, 0xffff };


/*
 * Draw an item title
 */

static void
DrawItemTitle (short item)
{
DispItem	*dp = &dispItem[item];

	MoveTo (titleHOffset, dp->vOffset);
	DrawString (dp->title);
}

/*
 * Draw an item value if it's different than the current value and update
 * the current value.  If alwaysDraw is true, draw/update it anyway.
 */
 
static void
DrawItemValue (short item, StringPtr value, Boolean alwaysDraw)
{
DispItem	*dp = &dispItem[item];
Rect	r;
short	h, v;

	if (alwaysDraw || !PStrEqual (dp->value, value))
	{
		PStrCpy (dp->value, value);
		h = titleHOffset + StringWidth (dp->title) + 10;
		v = dp->vOffset;
		SetRect (&r, h, v - 16, statusWind->portRect.right, v + 4);
		EraseRect (&r);
		MoveTo (h, v);
		DrawString (value);
	}
}


static void
HideBar (void)
{
	EraseRect (&barRect);
}


static void
StartBar (void)
{
Rect		r;
RGBColor	foreColor;

	r = barRect;
	FrameRect (&r);
	InsetRect (&r, 1, 1);
	/*
	 * Paint the bar the background color if drawing in color
	 * (else leave it white).
	 */
	if (useColorBar)
	{
		GetForeColor (&foreColor);
		RGBForeColor (&barBackGnd);
		PaintRect (&r);
		RGBForeColor (&foreColor);
	}
	else
		EraseRect (&r);
}


static void
PaintBar (void)
{
Rect		r;
RGBColor	foreColor;

	r = barRect;
	InsetRect (&r, 1, 1);
	r.right = r.left + ((r.right - r.left) * percent) / 100;
	if (useColorBar)
	{
		GetForeColor (&foreColor);
		RGBForeColor (&barForeGnd);
	}
	PaintRect (&r);
	if (useColorBar)
		RGBForeColor (&foreColor);
}


static pascal void
Mouse (Point pt, long when, short modifiers)
{
WindowPtr		w;
ControlHandle	ctrl;

	GetPort ((GrafPtr *) &w);
	if (FindControl (pt, w, &ctrl)
			&& TrackControl (ctrl, pt, nil)
			&& ctrl == cancelBtn)
	{
		EndTranslation (abortTranslation);
	}
}


static pascal void
Key (short c, short code, short modifiers)
{
	if (appState->processingDocument && SkelCmdPeriod (SkelGetCurrentEvent ()))
	{
		if (cancelBtn != (ControlHandle) nil)
			SkelFlashButton (cancelBtn);
		EndTranslation (abortTranslation);
	}
}


static pascal void
Update (Boolean resized)
{
short		i;

	EraseRect (&statusWind->portRect);
	DrawControls (statusWind);
	
	for (i = 0; i < maxDispItem; i++)
	{
		DrawItemTitle (i);
		DrawItemValue (i, dispItem[i].value, true);
	}

	if (!appState->processingDocument)
		HideBar ();
	else
	{
		StartBar ();
		PaintBar ();
	}
}


static pascal void
Activate (Boolean active)
{
short	hilite;

	if (cancelBtn != (ControlHandle) nil)
	{
		hilite = (appState->processingDocument ? normalHilite : dimHilite);
		if (!active)
			hilite = dimHilite;
		(void) SkelHiliteControl (cancelBtn, hilite);
	}
}


static pascal void
Clobber (void)
{
	DisposeWindow (statusWind);
	statusWind = (WindowPtr) nil;
}


/* ----------------------------------------------------------------- */

/*
 * Public routines
 */


void
StatusWindInit (void)
{
GDHandle	gDevice;
Str255	title;
Rect	r;

	/*
	 * If window hasn't been created, try to create it.
	 */

	if (statusWind == (WindowPtr) nil)
	{
		if (SkelQuery (skelQHasColorQD))
			statusWind = GetNewCWindow (statusWindRes, nil, (WindowPtr) -1L);
		else
			statusWind = GetNewWindow (statusWindRes, nil, (WindowPtr) -1L);
	
		if (statusWind != (WindowPtr) nil)
		{
			PStrCpy (title, ApplicationName ());
			PStrCat (title, "\p Status");
			SetWTitle (statusWind, title);
			SkelPositionWindow (statusWind, skelPositionOnParentDevice,
							FixRatio (1, 2), FixRatio (1, 5));
			}
			if (!SkelWindow (statusWind,
						Mouse,
						Key,
						Update,
						Activate,
						nil,
						Clobber,
						nil,
						true))
			{
				DisposeWindow (statusWind);
				statusWind = (WindowPtr) nil;
				return;
			}
	
			TextFont (0);
			TextSize (0);

			/*
			 * Create status bar.
			 */

			barRect = statusWind->portRect;
			InsetRect (&barRect, 10, 10);
			barRect.top = barRect.bottom - 12;

			/*
			 * Try to find the Cancel button (it's optional).  If it is, resize
			 * the window to make room for it, and resize/move the status bar
			 * a little.
			 */

			cancelBtn = GetNewControl (cancelCtrlRes, statusWind);
			if (cancelBtn != (ControlHandle) nil)
			{
				SizeWindow (statusWind,
							statusWind->portRect.right,
							statusWind->portRect.bottom + 3 + 8,
							false);
				r = (**cancelBtn).contrlRect;
				MoveControl (cancelBtn,
							statusWind->portRect.right - (r.right - r.left) - 13,
							statusWind->portRect.bottom - (r.bottom - r.top) - 13);
				barRect.right -= 3 + (r.right - r.left) + 13;
				OffsetRect (&barRect, 0, 4);	/* (20-12)/2 */
			}
	}

	percent = 0;

	/*
	 * Determine whether screen containing window can display bar in color.
	 * If the screen is set to black & white, both colors would end up drawing
	 * as black, so check for at least 4 colors (pixel depth 2 or greater).
	 * This will still fail if the user has  a multiple-monitor system and drags
	 * part or all of the meter bar to a black/white monitor during the
	 * translation, but that's too unlikely to bother messing with here.
	 */

	useColorBar = false;
	if (SkelQuery (skelQHasColorQD))
	{
		(void) SkelGetWindowDevice (statusWind, &gDevice, (Rect *) nil);
		if (gDevice != (GDHandle) NULL)
		{
			if ((**(**gDevice).gdPMap).pixelSize >= 2)
				useColorBar = true;
		}
	}

	StatusWindUpdate ();
	if (TstAppFlag (appInfo, persistentStatus))	/* show window if it's persistent */
	{
		SelectWindow (statusWind);
		ShowWindow (statusWind);
		SkelDoUpdates ();
	}
}


/*
 * Update the progress bar in the status window to reflect the current
 * translation percentage.  Update the document status values as well.
 *
 * The port needs to be saved, set, and restored because the status window
 * isn't necessarily the current port.
 *
 * Be sure to call StatusWindUpdate() after setting appState->processingDocument
 * but before actually reading the file so that it's called when percent = 0.
 * This makes sure the frame and background are drawn.
 */

void
StatusWindUpdate (void)
{
WindowPtr	w;
long		filePos;
long		num;
Str255		numBuf;
StringPtr	in, out;
short		hilite;

	if (statusWind == (WindowPtr) nil)
		return;

	GetPort (&w);
	SetPort (statusWind);
	/*
	 * Bar is empty if not processing document, otherwise it reflect
	 * progress through the current file.  Be careful to avoid divide-by-
	 * zero error on empty files.
	 */

	if (!appState->processingDocument)
		HideBar ();
	else
	{
		filePos = ftell (stdin);
		if (appState->inputSize == 0)
			percent = 100;
		else
			percent = (filePos * 100) / appState->inputSize;
		if (filePos == 0)
			StartBar ();			/* draw frame and background (empty bar) */
		if (percent > 0)
			PaintBar ();			/* paint status indication */
	}

	if (cancelBtn != (ControlHandle) nil)
	{
		hilite = (appState->processingDocument ? normalHilite : dimHilite);
		(void) SkelHiliteControl (cancelBtn, hilite);
	}

	/* update item values */

	NumToString ((long) appState->documentsProcessed, numBuf);
	DrawItemValue (processedItem, numBuf, false);
	num = PendingDocuments ();
	if (appState->processingDocument)
		++num;
	NumToString (num, numBuf);
	DrawItemValue (pendingItem, numBuf, false);
	if (!appState->processingDocument)
		in = out = "\p";
	else
	{
		in = appState->fssInput.name;
		if (appState->fssOutput.name[0] > 0)
			out = appState->fssOutput.name;
		else
			out = "\p(none)";
	}
	DrawItemValue (inputItem, in, false);
	DrawItemValue (outputItem, out, false);
	SetPort (w);

	/*
	 * If status window isn't persistent, make it go away when a document's
	 * done being processed.
	 */

	if (!TstAppFlag (appInfo, persistentStatus))
	{
		if (appState->processingDocument && !((WindowPeek) statusWind)->visible)
		{
			SelectWindow (statusWind);
			ShowWindow (statusWind);
		}
		else if (!appState->processingDocument && ((WindowPeek) statusWind)->visible)
			HideWindow (statusWind);
	}
}
