/*
 * html-mac.c -- Macintosh rtftohtml wrapper (for THINK C).
 *
 * Driver provides OpenOutputFile() and Basename() functions for the writer.
 */

# include	<stdio.h>

# include	"TransSkel.h"

# include	"rtf.h"
# include	"rtf-mac.h"
# include	"rtftohtml.h"
# include	"profile.h"


# define	prefsVersion	2


# define	optionsMenuRes		1002
# define	extDlogRes			128
# define	prefsDlogRes		129

# define	defInputSuffix		"\prtf"
# define	defOutputSuffix		"\phtml"
# define	defErrorSuffix		"\perr"
# define	defGraphicsSuffix	"\pgif"

# define	defOutputCreator	0L
# define	defErrorCreator		0L
# define	defGraphicsCreator	0L
# define	defGraphicsType		'PICT'

# define	defInlineGraphics	true
# define	defTableofContents	true
# define	defWriteGraphics	true

# define	normalHilite	0
# define	dimHilite		255


/*
 * Preferences structure.  Some of this information gets mapped into
 * the AppInfo structure so that the Macintosh wrapper can use it.
 * Some is specific to rtftohtml.  The routines in this file make
 * decisions based on the preferences structure, but map the relevant
 * information into the AppInfo structure because the wrapper makes
 * decisions based on that structure.
 *
 * The version number must ALWAYS be a short and the first member of
 * the structure.  This allows detection of obsolete versions of
 * preferences files.
 */

typedef	struct Prefs	Prefs;

struct Prefs
{
	short	version;				/* preferences version */

	Str31	outputSuffix;			/* output file suffix for autonaming */
	Str31	graphicsSuffix;			/* graphics file suffix for autonaming */
	Str31	errorSuffix;			/* error file suffix for autonaming */

	long	outputCreator;			/* creator of output files */
	long	graphicsCreator;		/* creator of graphics files */
	long	errorCreator;			/* creator of error files */
	long	graphicsType;			/* type of graphics files */

	Boolean	autoNameOutput;
	Boolean	autoNameError;

	Boolean	inlineGraphics;
	Boolean	TableofContents;
	Boolean	WriteGraphics;
};


typedef enum		/* Preferences dialog item numbers */
{
	iOK = 1,
	iCancel,
	iSave,
	iSuffixStaticText,
	iAutoNameStaticText,
	iTypeStaticText,
	iCreatorStaticText,
	iHTMLStaticText,
	iHTMLSuffixEditText,
	iHTMLYesRadioButton,
	iHTMLNoRadioButton,
	iHTMLTypeStaticText,
	iHTMLCreatorEditText,
	iHTMLSelectButton,
	iGraphicsStaticText,
	iGraphicsSuffixEditText,
	iGraphicsYesRadioButton,
	iGraphicsNoRadioButton,
	iGraphicsTypeStaticText,
	iGraphicsCreatorEditText,
	iGraphicsSelectButton,
	iErrorStaticText,
	iErrorSuffixEditText,
	iErrorYesRadioButton,
	iErrorNoRadioButton,
	iErrorTypeStaticText,
	iErrorCreatorEditText,
	iErrorSelectButton,
	iInlineCheckBox,
	iPrefsStaticText,
	iLineUserItem,
	iOKOutlineUserItem,
	iTOCCheckBox,
	iWGCheckBox
};


static void DriverInit (void);
static void DriverEnd (void);
static int	BeginFile ();
static void EndFile ();
static void DoPrefsDialog (void);


extern long	_ftype;
extern long	_fcreator;


static MenuHandle	optionsMenu;

static Prefs	*prefs = (Prefs *) NULL;


/*
 * Stuff to tell wrapper how to initialize/shutdown the writer, how to
 * begin/end processing for each output file, how to allow for
 * application-specific menu handling stuff, and default file suffixes.
 *
 * Some of this is filled in from the preferences file.
 */

static AppInfo	myAppInfo =
{
	nil,				/* no application-specific menu initialization */
	nil,				/* no application-specific menu adjustments */

	DriverInit,
	nil,

	BeginFile,
	EndFile,

	DoPrefsDialog,		/* preferences dialog */

	defInputSuffix,		/* input file suffix */
	"\p",				/* output file suffix (filled in from prefs) */
	"\p",				/* error file suffix (filled in from prefs) */
	
	0L,					/* output file creator (filled in from prefs) */
	0L,					/* error file creator (filled in from prefs) */

	/* flags (modified from prefs) */
	prefsInEditMenu
	| usesOutputFile
	| usesErrorFile
};

int
main (void)
{
	MacintoshWrapper (&myAppInfo);
}


/* ------------------------------------------------------------------- */


/*
 * Routines provided for use by writer code.
 */

/*
 * Open output file, setting type/creator appropriately for given
 * type of output file.  Relies on undocumented THINK C globals _ftype
 * and _fcreator.
 */

FILE *
OpenOutputFile (name, mode, fileType)
char	*name;
char	*mode;
int	fileType;	/* ignored */
{
FILE	*f;
long	oldftype;
long	oldfcreator;

	/* save current settings */
	oldftype = _ftype;
	oldfcreator = _fcreator;

	/* set type and creator.  fopen() uses these silently */
	switch (fileType)
	{
	default:
	case FTTEXT:
		_ftype = 'TEXT';
		_fcreator = myAppInfo.outputCreator;
		break;
	case FTPICT:
		_ftype = prefs->graphicsType;
		_fcreator = prefs->graphicsCreator;
		break;
	case FTWMF:
		_ftype = '????';
		_fcreator = prefs->graphicsCreator;	/* got a better guess? */
		break;
	case FTBMF:
		_ftype = '????';
		_fcreator = prefs->graphicsCreator;	/* got a better guess? */
		break;
	}

	if (_fcreator == 0L)
		_fcreator = docCreator;

	f = fopen (name, mode);

	/* restore previous settings */
	_ftype = oldftype;
	_fcreator = oldfcreator;

	return (f);
}


/*
 * Return pointer to basename part of pathname
 */


char *
Basename (name)
char	*name;
{
char	pathSep = PathSepChar ();
int	i, lsep;

	for(i=0,lsep=0;name[i]!='\0';i++)
	{
		if(name[i]==pathSep)
			lsep=i+1;
	}
	return (&name[lsep]);
}


/* ------------------------------------------------------------------- */


static Prefs *
GetPrefs (void)
{
char	*p;
long	size;

	if (ReadPrefs (&p, &size))
	{
		/* size must be at least enough for the version number at the front */
		if (size >= sizeof (short))
		{
			switch (* (short *) p)
			{
			case prefsVersion:		/* current version is the only one understood */
				if (size == sizeof (Prefs))
					return ((Prefs *) p);
				break;
			}
		}
		RTFFree ((char *) p);	/* it's bad, toss it */
	}
	return ((Prefs *) NULL);
}


/*
 * Map preferences information to AppInfo structure, where the two correspond.
 */

static void
MapPrefsToAppInfo (Prefs *prefs)
{
	PStrCpy (myAppInfo.outputSuffix, prefs->outputSuffix);
	PStrCpy (myAppInfo.errorSuffix, prefs->errorSuffix);

	myAppInfo.outputCreator = prefs->outputCreator;
	myAppInfo.errorCreator = prefs->errorCreator;

	if (prefs->autoNameOutput)
		myAppInfo.flags |= autoNameOutputFile;
	else
		myAppInfo.flags &= ~autoNameOutputFile;
	if (prefs->autoNameError)
		myAppInfo.flags |= autoNameErrorFile;
	else
		myAppInfo.flags &= ~autoNameErrorFile;
}


/*
 * Initialize driver.  This is called by the wrapper after the wrapper
 * has initialized itself.
 */

static void
DriverInit (void)
{
short	vRefNum;

	/*
	 * Read prefs file.  If that fails, initialize a preferences structure
	 * with the defaults.
	 */

	prefs = GetPrefs ();
	if (prefs != (Prefs *) NULL)
		RTFMsg ("prefs file opened for reading okay\n");
	else
	{
		RTFMsg ("prefs file could not be opened for reading, using defaults\n");

		if ((prefs = (Prefs *) RTFAlloc ((int) sizeof (Prefs))) == (Prefs *) NULL)
			RTFPanic ("Cannot allocate prefs structure.");

		prefs->version = prefsVersion;
	
		/*
		 * Preferences stuff that corresponds to standard AppInfo structure.
		 */
	
		PStrCpy (prefs->outputSuffix, defOutputSuffix);
		PStrCpy (prefs->errorSuffix, defErrorSuffix);
		PStrCpy (prefs->graphicsSuffix, defGraphicsSuffix);
	
		prefs->outputCreator = defOutputCreator;
		prefs->errorCreator = defErrorCreator;
	
		prefs->autoNameOutput = true;
		prefs->autoNameError = true;
	
		prefs->inlineGraphics = defInlineGraphics;
		prefs->TableofContents = defTableofContents;
		prefs->WriteGraphics = defWriteGraphics;
	
		/*
		 * Preferences that doesn't correspond to AppInfo structure.
		 */
	
		prefs->graphicsCreator = defGraphicsCreator;
		prefs->graphicsType = defGraphicsType;
	}

	MapPrefsToAppInfo (prefs);

	/*
	 * Done initializing driver, initialize writer
	 */

	WriterInit ();
}


/* ------------------------------------------------------------------- */


/*
 * The Macintosh wrapper passes the input file name and an output name.
 * Set up a fake argv array and pass it to do_main() to start the translation.
 * The input file has already been opened, and the error file is already
 * open, as infile.err.
 */

static int
BeginFile ()
{
int argc=1;
int i;
char *(argv[256]);
static char *input = (char *) NULL;
static char *output = (char *) NULL;
char	gExt[16];

	/* Get input/output file names.  Make copies because we modify them. (?) */
	RTFFree (input);
	RTFFree (output);
	input = RTFGetInputName ();
	if (input == (char *) NULL)
	{
		RTFMsg ("input file name unspecified\n");
		return (0);
	}
	input = RTFStrSave (input);
	output = RTFGetOutputName ();
	if (output == (char *) NULL)
	{
		RTFMsg ("output file name unspecified\n");
		return (0);
	}
	output = RTFStrSave (output);
	if (input == (char *) NULL || output == (char *) NULL)
	{
		RTFMsg ("could not allocate space for file names\n");
		return (0);
	}
	if(prefs->inlineGraphics)
		argv[argc++]="-i";
	if(!prefs->TableofContents)
		argv[argc++]="-T";
	if(!prefs->WriteGraphics)
		argv[argc++]="-G";
	/*
	 * The output name is either automatically generated, or else has
	 * already been solicited from the user.  In either case it's known.
	 */

	argv[argc++] = "-o";
	argv[argc++] = output;

	PStrCpy ((StringPtr) gExt, prefs->graphicsSuffix);
	(void) PtoCstr ((StringPtr) gExt);
	argv[argc++]="-P";
	argv[argc++]=gExt;

	argv[argc++]=input;

	return (do_main(argc, argv) == 1);	/* begin processing file */
}


static void
EndFile ()
{
	HTMLCleanup();
}


/* ------------------------------------------------------------------- */

/*
 * Preferences dialog routines
 */


static void
OSTypeToStr (OSType type, StringPtr str)
{
StringPtr	typeStr;

	typeStr = (StringPtr) &type;
	str[0] = 4;
	BlockMove (typeStr, &str[1], (long) 4);
}


static void
StrToOSType (StringPtr str, OSType *type)
{
short	i;

	for (i = 1; i <= 4; i++)
	{
		if (i <= str[0])
			((StringPtr) type)[i-1] = str[i];
		else
			((StringPtr) type)[i-1] = ' ';	/* blank fill on right */
	}
}


static void
SetOSType (DialogPtr dlog, short item, OSType type)
{
Str255		str;

	OSTypeToStr (type, str);
	SkelSetDlogStr (dlog, item, str);
	SelIText (dlog, item, 0, 32767);
}


static void
GetOSType (DialogPtr dlog, short item, OSType *type)
{
Str255		str;

	SkelGetDlogStr (dlog, item, str);
	StrToOSType (str, type);
}


static void
SelectApplication (DialogPtr dlog, short item)
{
FSSpec	fssApp;
FInfo	fndrInfo;

	if (SelectInputFile (&fssApp, 'APPL'))
	{
		if (FSpGetFInfo (&fssApp, &fndrInfo) != noErr)
			SysBeep (1);
		else
			SetOSType (dlog, item, fndrInfo.fdCreator);
	}
}


static pascal void
DrawUserLine (DialogPtr dlog, short item)
{
Rect	r;

	SkelGetDlogRect (dlog, item, &r);
	FrameRect (&r);
}


/*
 * Check text in suffix fields and hilite OK button according to whether
 * or not any are empty.  If the button changes state, invalidate the
 * surrounding rectangle so the outline gets redrawn, too.  The Save
 * button state follows that of the OK button.
 */

static pascal Boolean
PrefsFilter (DialogPtr dlog, EventRecord *evt, short *item)
{
Str255		str;
short		hilite;
Rect		r;

	hilite = normalHilite;
	SkelGetDlogStr (dlog, iHTMLSuffixEditText, str);
	if (str[0] == 0)
		hilite = dimHilite;
	SkelGetDlogStr (dlog, iGraphicsSuffixEditText, str);
	if (str[0] == 0)
		hilite = dimHilite;
	SkelGetDlogStr (dlog, iErrorSuffixEditText, str);
	if (str[0] == 0)
		hilite = dimHilite;
	if (SkelSetDlogCtlHilite (dlog, iOK, hilite))
	{
		SkelGetDlogRect (dlog, iOKOutlineUserItem, &r);
		InvalRect (&r);
		(void) SkelSetDlogCtlHilite (dlog, iSave, hilite);
	}
	return (false);
}


static void
DoPrefsDialog (void)
{
ModalFilterProcPtr	filter;
DialogPtr	dlog;
Boolean		dlogDone;
short		item;
GrafPtr		savePort;
Str255		str;
OSType		creator;

	dlog = GetNewDialog(prefsDlogRes, nil, (WindowPtr) -1L);
	if (dlog == (DialogPtr) nil)
	{
		SysBeep (1);
		return;
	}
	SkelPositionWindow (dlog, skelPositionOnParentDevice,
									FixRatio (1, 2), FixRatio (1, 5));

	SkelSetDlogButtonOutliner (dlog, iOKOutlineUserItem);
	SkelSetDlogProc (dlog, iLineUserItem, DrawUserLine);

	SkelSetDlogStr (dlog, iHTMLSuffixEditText, prefs->outputSuffix);
	SkelSetDlogStr (dlog, iGraphicsSuffixEditText, prefs->graphicsSuffix);
	SkelSetDlogStr (dlog, iErrorSuffixEditText, prefs->errorSuffix);

	item = (prefs->autoNameOutput ? iHTMLYesRadioButton : iHTMLNoRadioButton);
	SkelSetDlogRadioButtonSet (dlog, iHTMLYesRadioButton, iHTMLNoRadioButton, item);
	item = (prefs->autoNameError ? iErrorYesRadioButton : iErrorNoRadioButton);
	SkelSetDlogRadioButtonSet (dlog, iErrorYesRadioButton, iErrorNoRadioButton, item);

	/* graphics files are always autonamed; handle buttons differently */
	item = iGraphicsYesRadioButton;
	SkelSetDlogRadioButtonSet (dlog, iGraphicsYesRadioButton, iGraphicsNoRadioButton, item);
	(void) SkelSetDlogCtlHilite (dlog, iGraphicsNoRadioButton, dimHilite);

	creator = prefs->outputCreator;
	if (creator == 0L)
		creator = docCreator;
	SetOSType (dlog, iHTMLCreatorEditText, creator);
	creator = prefs->graphicsCreator;
	if (creator == 0L)
		creator = docCreator;
	SetOSType (dlog, iGraphicsCreatorEditText, creator);
	creator = prefs->errorCreator;
	if (creator == 0L)
		creator = docCreator;
	SetOSType (dlog, iErrorCreatorEditText, creator);

	SkelSetDlogCtlValue (dlog, iInlineCheckBox, prefs->inlineGraphics);
	SkelSetDlogCtlValue (dlog, iTOCCheckBox, prefs->TableofContents);
	SkelSetDlogCtlValue (dlog, iWGCheckBox, prefs->WriteGraphics);

	/* select first edit text item */
	SelIText (dlog, iHTMLSuffixEditText, 0, 32767);

	GetPort (&savePort);
	SetPort (dlog);
	ShowWindow(dlog);
	
	dlogDone=false;
	while(!dlogDone)
	{
		filter = SkelDlogFilter (PrefsFilter, true);
		SkelDlogDefaultItem (iOK);
		SkelDlogCancelItem (iCancel);
		SkelDlogTracksCursor (true);
		ModalDialog(filter,&item);
		SkelRmveDlogFilter ();

		switch(item)
		{
			case iOK:
			case iCancel:
			case iSave:
				dlogDone=true;
				break;

			case iHTMLYesRadioButton:
			case iHTMLNoRadioButton:
				SkelSetDlogRadioButtonSet (dlog,
							iHTMLYesRadioButton, iHTMLNoRadioButton, item);
				break;

			case iErrorYesRadioButton:
			case iErrorNoRadioButton:
				SkelSetDlogRadioButtonSet (dlog,
							iErrorYesRadioButton, iErrorNoRadioButton, item);
				break;

			case iHTMLSelectButton:
				SelectApplication (dlog, iHTMLCreatorEditText);
				break;
			case iGraphicsSelectButton:
				SelectApplication (dlog, iGraphicsCreatorEditText);
				break;
			case iErrorSelectButton:
				SelectApplication (dlog, iErrorCreatorEditText);
				break;

			case iInlineCheckBox:
				SkelToggleDlogCtlValue (dlog, iInlineCheckBox);
				break;
			case iTOCCheckBox:
				SkelToggleDlogCtlValue (dlog, iTOCCheckBox);
				break;
			case iWGCheckBox:
				SkelToggleDlogCtlValue (dlog, iWGCheckBox);
				break;
		}
	}

	if (item == iOK || item == iSave)
	{
		SkelGetDlogStr (dlog, iHTMLSuffixEditText, str);
		if (str[0] > 31)
			str[0] = 31;
		PStrCpy (prefs->outputSuffix, str);
		SkelGetDlogStr (dlog, iGraphicsSuffixEditText, str);
		if (str[0] > 31)
			str[0] = 31;
		PStrCpy (prefs->graphicsSuffix, str);
		SkelGetDlogStr (dlog, iErrorSuffixEditText, str);
		if (str[0] > 31)
			str[0] = 31;
		PStrCpy (prefs->errorSuffix, str);

		prefs->autoNameOutput = SkelGetDlogCtlValue (dlog, iHTMLYesRadioButton);
		prefs->autoNameError = SkelGetDlogCtlValue (dlog, iErrorYesRadioButton);

		GetOSType (dlog, iHTMLCreatorEditText, (OSType *) &prefs->outputCreator);
		GetOSType (dlog, iGraphicsCreatorEditText, (OSType *) &prefs->graphicsCreator);
		GetOSType (dlog, iErrorCreatorEditText, (OSType *) &prefs->errorCreator);

		prefs->inlineGraphics = SkelGetDlogCtlValue (dlog, iInlineCheckBox);
		prefs->TableofContents = SkelGetDlogCtlValue (dlog, iTOCCheckBox);
		prefs->WriteGraphics = SkelGetDlogCtlValue (dlog, iWGCheckBox);

		/* now remap prefs onto the AppInfo struct */

		MapPrefsToAppInfo (prefs);
	}
	if (item == iSave)
	{
		if (WritePrefs ((char *) prefs, sizeof (Prefs)))
			RTFMsg ("prefs file opened for writing okay\n");
		else
			RTFMsg ("prefs file could not be opened for writing\n");
	}

	SetPort (savePort);
	DisposeDialog(dlog);
}
